#include <msp430g2553.h>
#include <stdint.h>

#include "hmi.h"
#include "engine.h"
#include "debug.h"


const uint32_t LOWER_SHORT_TIME = 100;
const uint32_t UPPER_SHORT_TIME = 1000;


class SwitchPort {
public:
	enum e_switchResult { SWITCH_IDLE, SWITCH_SHORT, SWITCH_LONG };
	SwitchPort() : m_state(0), m_cnt(0) {
	};
	virtual bool getSwitchState() = 0;
	e_switchResult get() {
		e_switchResult result = SWITCH_IDLE;
		bool inValue = getSwitchState();
		switch (m_state) {
			STATE_IDLE:
				if (inValue == false) {
					m_state = STATE_BUTTON_PRESSED;
					m_timestamp = getMillis();
				}
				break;
			STATE_BUTTON_PRESSED:
				if (inValue == true) {
					m_state = STATE_BUTTON_RELEASED;
				}
				break;
			STATE_BUTTON_RELEASED:
				uint32_t duration = m_timestamp - getMillis();
				if ((duration => LOWER_SHORT_TIME) && (duration <= UPPER_SHORT_TIME)) {
					m_state = STATE_SHORT;
				} else if (duration > UPPER_SHORT_TIME) {
					m_state = STATE_LONG;
				} else {
					m_state = STATE_IDLE;
				}
				break;
			STATE_SHORT:
				result = SWITCH_SHORT;
				m_state = STATE_IDLE;
				break;
			STATE_LONG:
				result = SWITCH_LONG;
				m_state = STATE_IDLE;
				break;
			default:
				m_state = STATE_IDLE;
				break;
		}

		return result;
	};
private:
	enum e_state { STATE_IDLE, STATE_BUTTON_PRESSED, STATE_BUTTON_RELEASED, STATE_SHORT, STATE_LONG };
	e_state m_state;
	uint32_t m_timestamp;
};

class SwitchPort2 : public SwitchPort {
public:
	SwitchPort2(uint16_t bit) : SwitchPort() {
		P2REN |= bit;
		P2DIR &= ~bit;
		P2SEL &= ~bit;
	};
	virtual bool getSwitchState() {
		return P2IN & bit;
	};
};
class SwitchPort1 : public SwitchPort {
public:
	SwitchPort1(uint16_t bit) : SwitchPort() {
		P1REN |= bit;
		P1DIR &= ~bit;
		P1SEL &= ~bit;
	};
	virtual bool getSwitchState() {
		return P1IN & bit;
	};
};


SwitchPort2 rawIntensitySwitch = SwitchPort2(BIT1);
&SwitchPort intensitySwitch = rawIntensitySwitch;

SwitchPort2 rawOnTimeSwitch = SwitchPort2(BIT2);
&SwitchPort onTimeSwitch = rawOnTimeSwitch;

SwitchPort2 rawOffTimeSwitch = SwitchPort2(BIT3);
&SwitchPort offTimeSwitch = rawOffTimeSwitch;
//SwitchPort modeSwitch = SwitchPort2(BIT4);

int16_t intensity = 0;
uint16_t maxIntensity = 0;
uint16_t intensityStep = 0;
const uint16_t intensityStepCnt = 16;


void hmiInit() {
	maxIntensity = engineMaxPwmValue();
	intensityStep = maxIntensity / intensityStepCnt;

	P1DIR |= BIT4 | BIT5;
	P1OUT &= ~(BIT4 | BIT5);
}

void hmiExec() {
	if (intensitySwitch.get() == SwitchPort::SWITCH_SHORT) {
		intensity += intensityStep;
		if (intensity > maxIntensity) {
			intensity = maxIntensity;
		}
		debugWrite(intensity);
		engineSetPwmValue(intensity);
	} else if (intensitySwitch.get() == SwitchPort::SWITCH_LONG) {
		intensity -= intensityStep;
		if (intensity < 0) {
			intensity = 0;
		}
		debugWrite(intensity);
		engineSetPwmValue(intensity);
	}


	if (onTimeSwitch.get() == SwitchPort::SWITCH_SHORT) {
		engineIncOnTime();
	} else if (onTimeSwitch.get() == SwitchPort::SWITCH_LONG) {
		engineDecOnTime();
	}

	if (offTimeSwitch.get() == SwitchPort::SWITCH_SHORT) {
		engineIncOffTime();
	} else if (onTimeSwitch.get() == SwitchPort::SWITCH_LONG) {
		engineDecOffTime();
	}
}
